/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "config.h"
#include <librnd/core/error.h>
#include <librnd/core/compat_misc.h>

#include "obj_any.h"

#include "drc.h"

unsigned long osm_drc_cnt;
vtL0_t *drc_list;

void osm_drc_error(osm_any_obj_t *obj, const char *msg)
{
	rnd_message(RND_MSG_ERROR, "DRC error: %s #" OSM_PID ": %s\n",
		osm_obj_name[obj->hdr.type], obj->hdr.id, msg);

	if (drc_list != NULL) {
		osm_objlist_item_t *item = vtL0_alloc_append(drc_list, 1);
		item->type = obj->hdr.type;
		item->id = obj->hdr.id;
		item->comment = rnd_strdup(msg);
	}

	osm_drc_cnt++;
}

void osm_drc_clear_list(vtL0_t *lst)
{
	long n;

	for(n = 0; n < lst->used; n++)
		free(lst->array[n].comment);

	vtL0_uninit(lst);
}
