/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#define _XOPEN_SOURCE


#include "config.h"

#include <librnd/config.h>
#include <librnd/core/unit.h>
#include <librnd/core/rnd_printf.h>
#include <librnd/core/error.h>

#include "projection.h"

#include "osm_printf.h"

rnd_aft_ret_t RND_AFT_osm_coord = RND_AFT_error;

rnd_aft_ret_t osm_printf_app_format(gds_t *string, gds_t *spec, const char **fmt, rnd_unit_allow_t mask, rnd_unit_suffix_t suffix, rnd_aft_arg_t *arg, rnd_aft_arg_t *cookie)
{
	rnd_coord_t c;
	double d;

	if (RND_AFT_osm_coord == RND_AFT_error) {
		if (sizeof(int)== sizeof(rnd_coord_t))
			RND_AFT_osm_coord = RND_AFT_INT;
		else if (sizeof(long) == sizeof(rnd_coord_t))
			RND_AFT_osm_coord = RND_AFT_LONG;
		else {
			rnd_message(RND_MSG_ERROR, "Can't find matching integer type for sizeof(rnd_coord_t)\n");
			return RND_AFT_error;
		}
	}

	if (arg == NULL) return RND_AFT_osm_coord;
	c = (RND_AFT_osm_coord == RND_AFT_INT) ? arg->i : arg->l;

	switch(**fmt) {
		case 'a':
			d = rnd2latdeg(c);
			goto print_it;
		case 'o':
			d = rnd2londeg(c);
			print_it:;
			if (rnd_append_printf(string, "%.7f", (double)d) < 0)
				return RND_AFT_error;
			return RND_AFT_osm_coord;

		case 'm':
			if (rnd_append_printf(string, "%.2f", (double)c/100.0) < 0)
				return RND_AFT_error;
			return RND_AFT_osm_coord;
	}
	return RND_AFT_error;
}
