/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - server communication
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "obj_any.h"

RND_INLINE void history_get_lyobj(osm_history_t *hst, vtp0_t *work, osm_objtype_t type, osm_id_t id, gds_t *tmp)
{
	char tid[64], *tmpfn;

	rnd_trace("Hl: get %s:%ld\n", osm_obj_name[type], id);

	tmp->used = 0;
	gds_append_str(tmp, "https://");
	gds_append_str(tmp, conf_net_osm_xml.plugins.net_osm_xml.hostname);
	gds_append_str(tmp, "/api/0.6/");
	gds_append_str(tmp, osm_obj_name[type]);
	gds_append(tmp, '/');
	sprintf(tid, OSM_PID, id);
	gds_append_str(tmp, tid);
	gds_append_str(tmp, "/history");

	rnd_trace("     url='%s'\n", tmp->array);

	tmpfn = rnd_tempfile_name_new("osm_hst.xml");
	osm_wget_disk(tmp->array, tmpfn, 0, NULL);

	osm_oxml_load_history(hst, tmpfn);

	rnd_tempfile_unlink(tmpfn);
}

RND_INLINE void history_get_changeset(osm_history_t *hst, vtp0_t *work, osm_id_t chs_id, gds_t *tmp)
{
	char tid[64], *tmpfn;

	if (htIp_get(&hst->id2obj[OSM_OBJ_CHANGESET], chs_id) != NULL)
		return;

	rnd_trace("Hc: get changeset:%ld\n", chs_id);

	tmp->used = 0;
	gds_append_str(tmp, "https://");
	gds_append_str(tmp, conf_net_osm_xml.plugins.net_osm_xml.hostname);
	gds_append_str(tmp, "/api/0.6/changeset/");
	sprintf(tid, OSM_PID, chs_id);
	gds_append_str(tmp, tid);

	rnd_trace("     url='%s'\n", tmp->array);

	tmpfn = rnd_tempfile_name_new("osm_hst.xml");
	osm_wget_disk(tmp->array, tmpfn, 0, NULL);

	osm_oxml_load_changeset(hst, tmpfn);

	rnd_tempfile_unlink(tmpfn);
}


RND_INLINE void history_get_obj(osm_history_t *hst, vtp0_t *work, osm_any_obj_t *obj, gds_t *tmp)
{
	if (obj->hdr.type < OSM_OBJ_max)
		history_get_lyobj(hst, work, obj->hdr.type, obj->hdr.id, tmp);
}


RND_INLINE void history_add_referees(osm_history_t *hst, vtp0_t *work, osm_any_obj_t *obj, gds_t *tmp)
{
	long n, m, i;
	vtp0_t *lst2, *lst = htIp_get(&hst->id2obj[obj->hdr.type], obj->hdr.id);

	assert(lst != NULL);

	switch(obj->hdr.type) {
		case OSM_OBJ_WAY:
			for(m = 0; m < lst->used; m++) { /* check all versions of the way */
				obj = lst->array[m];
				if (obj->hdr.changeset > 0)
					history_get_changeset(hst, work, obj->hdr.changeset, tmp);
				for(n = 0; n < obj->way.curr->nd.used; n++) { /* check all nodes of this version of the way */
					osm_id_t node_id = obj->way.curr->nd.array[n];
					if (htIp_get(&hst->id2obj[OSM_OBJ_NODE], node_id) == NULL) {
						history_get_lyobj(hst, work, OSM_OBJ_NODE, node_id, tmp);

						/* fetch changelog for all vesions of the new node */
						lst2 = htIp_get(&hst->id2obj[OSM_OBJ_NODE], node_id);
						for(i = 0; i < lst2->used; i++) {
							osm_any_obj_t *ndo = lst2->array[i];
							history_get_changeset(hst, work, ndo->hdr.changeset, tmp);
						}
					}
				}
			}
			break;
		case OSM_OBJ_RELATION:
			TODO("relation: implement this");
			break;
		case OSM_OBJ_NODE:
		case OSM_OBJ_CHANGESET:
		case OSM_OBJ_invalid:
		case OSM_OBJ_vmax:
			break;
	}
}

static long history_get(osm_history_t *hst, vtp0_t *work)
{
	long n, cnt = 0;
	gds_t tmp = {0};

	for(n = 0; n < work->used; n++) {
		osm_any_obj_t *obj = work->array[n];
		if ((obj != NULL) && (htIp_get(&hst->id2obj[obj->hdr.type], obj->hdr.id) == NULL)) {
			history_get_obj(hst, work, obj, &tmp);
			history_add_referees(hst, work, obj, &tmp);
			if (obj->hdr.changeset > 0)
				history_get_changeset(hst, work, obj->hdr.changeset, &tmp);
			cnt++;
		}
	}

	gds_uninit(&tmp);

	return cnt;
}

